@@ -9,6 +9,8 @@ import operator
 import random
 import string
 from memoize import Memoize
+import numpy
+import util
 
 def collect(exp, fn):
   rv = set()
@@ -72,6 +74,10 @@ class _Symbolic(tuple):
       #  oldexp = exp
       #  exp = exp.walk(fn)
 
+    if util.DEBUG and exp != self:
+      #print '%s => %s (%s)' % (self, exp, fn)
+      pass
+     
     return exp
 
   def _dump(self):
@@ -80,6 +86,18 @@ class _Symbolic(tuple):
         'id': id(self)
         }
 
+  def __contains__(self, exp):
+    rv = {}
+    rv['val'] = False
+
+    def _(_exp):
+      if _exp.match(exp):
+        rv['val'] = True
+      return _exp
+    self.walk(_)
+
+    return rv['val']
+
   def substitute(self, subs):
     '''
     takes a dictionary of substitutions
@@ -90,6 +108,19 @@ class _Symbolic(tuple):
 
     return self
 
+  def compile(self, *arguments):
+    '''compiles a symbolic expression with arguments to a python function'''
+
+    def _compiled_func(*args):
+      assert len(args) == len(arguments)
+      argdic = {}
+      for i in range(len(args)):
+        argdic[arguments[i]] = args[i]
+      rv = self.substitute(argdic).simplify()
+      return desymbolic(rv)
+
+    return _compiled_func
+
   def __eq__(self, other):
     #return type(self) == type(other) and self.name == other.name
     return id(self) == id(other)
@@ -226,6 +257,10 @@ class Number(_KnownValue):
     self.n = n
     return self
 
+  @property
+  def is_integer(self):
+    return self.n.is_integer()
+
   def value(self):
     return self.n
 
@@ -273,6 +308,18 @@ class WildResults(object):
   def __getattr__(self, idx):
     return self[idx]
 
+  def __iter__(self):
+    return self._hash.__iter__()
+
+  def __str__(self):
+    return str(self._hash)
+
+  def __repr__(self):
+    return str(self)
+
+  def __len__(self):
+    return len(self._hash)
+
 class Wild(_Symbolic):
   '''
   wilds will be equal to anything, and are used for pattern matching
@@ -395,6 +442,14 @@ class Fn(_Symbolic):
 
     return self
 
+  def recursive_substitute(self, subs):
+    y = self
+    while True:
+      x = y.substitute(subs)
+      if x == y:
+        return x
+      y = x
+
   def __getitem__(self, n):
     if n == 0:
       return self.fn
@@ -536,7 +591,7 @@ def symbolic(obj, **kargs):
   '''
   makes the symbolic version of an object
   '''
-  if type(obj) in [type(0), type(0.0), type(0L)]:
+  if type(obj) in [type(0), type(0.0), type(0L), numpy.int32]:
     return Number(obj, **kargs)
   elif type(obj) == type('str'):
     return Symbol(obj, **kargs)
@@ -548,4 +603,18 @@ def symbolic(obj, **kargs):
     msg = "Unknown type (%s) %s passed to symbolic" % (type(obj), obj)
     raise BaseException(msg)
 
+def desymbolic(s):
+  '''
+  returns a numeric version of s
+  '''
+
+  if type(s) in (int,long,float):
+    return s
+
+  s = s.simplify()
+  if not isinstance(s, Number):
+    raise BaseException("Only numbers can be passed to desymbolic")
+
+  return s.value()
+
 import stdops
